/*
 * Message Digest Cipher as described in:
 *   Bruce Schneier: Applied Cryptography. John Wiley & Sons, 1992.
 *
 * Copyright (c) y2k Micha Moskal <malekith@topnet.pl>
 *
 * This file is part of libcftp - cftp protocol implementation.
 * CryptoFTP is free software. You can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License (see COPYING.LGPL 
 * for details).
 *
 * $Id: mdc.c,v 1.1.1.1 2000/04/22 10:15:52 malekith Exp $
 */
 
#include "config.h"
#include "utils.h"
#include "cipher.h"

#include <string.h>

#include "md5.inc"

/* remember that entire pool_size should be covered by hash computing function 
 * also POOL_SIZE must be power of 2
*/
#define POOL_SIZE 64
#define RESULT_SIZE 16

struct mdc_ctx {
	struct cipher parent;
	
	struct md5_ctx md5;
	byte pool[POOL_SIZE];
	int pos;
};


static void mdc_get(struct mdc_ctx *ctx, void *target)
{
	md5_process_block(&ctx->md5, ctx->pool);
	md5_read_ctx(&ctx->md5, target);
	memcpy(ctx->pool + ctx->pos, target, RESULT_SIZE);
	ctx->pos += RESULT_SIZE;
	ctx->pos &= POOL_SIZE - 1;
}

static void mdc_init_key(struct mdc_ctx *ctx, const void *key, int keylen)
{
	byte dummy[RESULT_SIZE];
	int n;
	
	memset(ctx->pool, 0, POOL_SIZE);
	ctx->pos = 0;
	memcpy(ctx->pool, key, POOL_SIZE < keylen ? POOL_SIZE : keylen);
	
	md5_init_ctx(&ctx->md5);
	
	n = 128;
	
	while (n--)
		mdc_get(ctx, dummy);
}

static void mdc_cipher_block(struct mdc_ctx *ctx, byte *block, int len)
{
	u32 buf[RESULT_SIZE / 4];
	u32 *p;
	int i;
	
	if (!len)
		return;
		
	len /= RESULT_SIZE;
	p = (u32 *)block;
	
	while (len--) {
		mdc_get(ctx, buf);
		
		for (i = 0; i < RESULT_SIZE / 4; i++)
			*p++ ^= buf[i++];
	}
}

static void dfree(struct cipher *self)
{
	xfree0(self, sizeof(struct mdc_ctx));
}

struct cipher *mdc_create()
{
	struct cipher *r;
	r = (struct cipher *)xmalloc(sizeof(struct mdc_ctx));
	r->init = (init_cipher_func)mdc_init_key;
	r->cipher = (cipher_func)mdc_cipher_block;
	r->decipher = (cipher_func)mdc_cipher_block;
	r->free = dfree;
	
	r->keylen = POOL_SIZE;
	r->blocklen = RESULT_SIZE;
	r->name = "mdc-md5";
	r->disabled = 0;
	return r;
}
