/*
 * Copyright (c) y2k Micha Moskal <malekith@topnet.pl>
 *
 * This file is part of libcftp - cftp protocol implementation.
 * CryptoFTP is free software. You can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License (see COPYING.LGPL 
 * for details).
 *
 * $Id: protocol.h,v 1.4 2000/04/23 23:07:41 malekith Exp $
 */

struct packet {
	i32 size;  /* of data + 6 */
	u32 crc32; /* of data [only, unen */
	byte id;
	byte pad; /* size of padding in data */
	
	byte data[1];
};

/* ID: */
enum {
	PKT_ERROR = 0,
	PKT_IGNORE,
	PKT_DISCONNECT,
	PKT_RSAKEY,
	PKT_KEY,
	PKT_KEY_RSP,
	PKT_COOKIE,
	PKT_AUTH,
	PKT_ACCESS_DENIED,
	PKT_MSG,
	PKT_COMMAND,
	PKT_RESPONSE,
	PKT_FILE,
	PKT_EOF,
	PKT_ABORT,
	PKT_ZFILE,
	PKT_FORWARD_AUTH,
	PKT_FORWARD_DATA,
	PKT_FORWARD_EOF,
	PKT_FORWARD_CONNECT,
};

enum {
	CMD_RESERVED = 0,
	CMD_INFO = 1, 	/* returns human-readable general info */
	CMD_PWD,
	CMD_CD,		/* accepts ".." */
	CMD_LS,
	CMD_STOR,
	CMD_GET,
	CMD_RM,
	CMD_MKDIR,
	CMD_RMDIR,
	CMD_RMR,	/* rm -r */
	CMD_MV,
	CMD_FORWARD,
};

#define COMMAND_NAMES \
	"RESERVED", "INFO", "PWD", "CD", "LS", "STOR", "GET", "RM", "MKDIR", \
	"RMDIR", "RMR", "MV", "FORWARD"

enum {
	RSP_RESERVED = 0,
	/* errors: */
	RSP_ERROR = 1,
	RSP_PROTOCOL_VIOLATION,
	RSP_INTERNAL, /* some error inside server */
	RSP_NO_SUCH_COMMAND,
	RSP_TOO_MUCH_ARGS,
	
	/* status from commands: */
	RSP_STATUS_OK,
	RSP_STATUS_ERR,
	RSP_ABORTED,
	
	/* info: */
	RSP_CWD,
	RSP_INFO,
	RSP_LS,
	RSP_GET,
	
	/* packets which can be send as an answer to PKT_EOF */	
	RSP_EOF_OK,
	/* donates violation of protocol: */	
	RSP_EOF_NO_FILE,
	RSP_EOF_TOO_MANY,
	RSP_EOF_TOO_FEW, 
	/* donates some real problems on server (or quota ;) */	
	RSP_EOF_WRITE_ERROR,
	RSP_EOF_ABORTED,
	
	RSP_FORWARD,
	
	/* flags start here */	
	RSP_READ_ON = 0x1000, /* used for multipacket responses */
};

struct pkt_command {
	i32 no;
	byte data[1];
};

struct pkt_response {
	i32 no;
	byte data[1];
};

/*
 * this packet is sent encoded as mpz_t and encrypted by server RSA key
 * length of this struct is 640 bits (80 bytes) so minimal RSA key length
 * is 640 bits (use better 768 bits)
 */
struct pkt_key {
	byte cipher_id[10];	/* nul-padded ascii cipher short name */
	byte zero;		/* always zero */
	byte used_key_len;	/* actually used bytes of server,client_key */
	byte server_key[32];	/* key use for encryption in server */
	byte client_key[32];	/* key use for encryption in client */
};

struct pkt_key_rsp {
	/* 1 if server has accepted cipher and keys, 0 otherwise */
	byte accepted;
	/* NUL terminated message, should be displayed to user, especially
	 * if accepted == 0, it can contain some reason why given cipher 
	 * is not supported/disabled */
	byte msg[1]; 
} ;

/*
 * 64bits sizes will be included in 64bit version of these structs someday.
 */

/*
 * used by RSP_LS
 */
struct rsp_ls {
	/*
	 * MODE is defined:
	 * 0007777 is permision mask (drwxrwxrwx) with suid bits
	 * 0007000 this mask is used for setuid, setgid and sticky bit
	 *   filesystems not supporting more then one user should use 
	 *   OWNER permisions (with mask == 00700)
	 * Higher bits AREN'T same as these from <sys/stat.h> !
	 * these are defined below
	 * MODE == -1 marks end of ls block
	 */
	u32 mode;
	u32 mtime;	/* modification time in seconds since Epoch */
	u32 size;	/* size of file in bytes */
	u32 nlink;	/* for human reader only, don't use in client's code */
	byte data[1];	/* user, group, file name and possibly symlink target 
			 * (if (mode & MODE_SYMLINK)) as NUL terminated 
			 * strings */
} ;
enum {
	MODE_DIRECTORY 	= 010000,
	MODE_SYMLINK 	= 020000,
};

/*
 * used by CMD_GET
 */
struct cmd_get {
	u32 flags;		/* falgs defined below */
	u32 start_at;	/* offset to start at */
	u32 nbytes;		/* number of bytes to store */
	byte name[1];		/* NUL terminated file name */
} ;
enum {
	GET_WHOLE	= 0x00000001, /* nbytes is ignored, entire file is retrived */
 	GET_DEFLATE 	= 0x00000002, 
};
/*
 * sent back as RSP_GET in answer to CMD_GET
 */
struct rsp_get {
	u32 mode; 	/* permisions (mask: 07777) */
	/* number of bytes that will be send (for progress bars, etc): */
	u32 size;
	byte data[1]; /* user & group as NUL terminated strings */
} ;

/*
 * used by CMD_STOR
 */
struct cmd_stor {
	u32 flags;	/* flags as defined below */
	u32 start_at;	/* offset to start at */
	u32 nbytes;	/* number of bytes to store */
	u32 mode; 	/* regular unix mode, with mask 07777 */
	byte name[1];	/* NUL terminated file name */
};
enum {
	STOR_TRUNC	= 0x00000001, /* null file first */
	STOR_EXCL	= 0x00000002, /* don't clobber existing file */
	STOR_APPEND	= 0x00000004, /* if used, start_at is ignored */
};


struct cmd_forward {
	u32 flags;		/* unused -- set to 0 */
	u32 port;		/* u32, not u16 for simplicity */
	byte server_key[32];	/* keys used for encryption */
	byte client_key[32];	/* cipher & used_key_len are the same... */
	/* probably ignored by server ... if host[0] == 0, connection
	 * to localhost is made */
	byte host[1];
};

struct rsp_forward {
	u32 flags;		/* unused -- set to 0 */
	u32 port;		/* this is remote port, for client to connect */
};
